package Asterisk::API::Documentation::Generic;

use strict;
use warnings;

our $VERSION = '1';

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    return $self;
}

sub prepare_types {
    my ( $self, $doc ) = @_;

    my $types;

    foreach my $type ( @{ $doc->{types} } ) {
	my $type_item;

	$type_item->{type} = $type->{type};

	foreach my $key ( sort keys %{ $type->{structure} } ) {
	    my $key_type = ref $type->{structure}->{$key}; # Types ( ref($arg) result ): "" - scalar, "HASH" - hash, "ARRAY" - array

	    if ( $key_type eq "" ) {
		if ( length $type->{structure}->{$key} ) {
		    push @{ $type_item->{items} }, { item => $type->{structure}->{$key}, props => {
			type => "string", format => "raw", description => "", required => 0, hidden => 0, transform => $key } };
		}
		else {
		    push @{ $type_item->{items} }, { item => $type->{structure}->{$key}, props => { type => "string", format => "raw", description => "", required => 0, hidden => 0, transform => $key } };
		}
	    }
	    elsif ( $key_type eq "HASH" ) {
		next if exists $type->{structure}->{$key}->{hidden} && $type->{structure}->{$key}->{hidden};
		if ( exists $type->{structure}->{$key}->{transform} && ref $type->{structure}->{$key}->{transform} eq "ARRAY" ) {
		    foreach my $transform_key ( @{ $type->{structure}->{$key}->{transform} } ) {
			push @{ $type_item->{items} }, { item => $transform_key, props => $type->{structure}->{$key} };
		    }
		}
		elsif ( exists $type->{structure}->{$key}->{transform} && length $type->{structure}->{$key}->{transform} ) {
		    push @{ $type_item->{items} }, { item => $type->{structure}->{$key}->{transform}, props => $type->{structure}->{$key} };
		}
		else {
		    push @{ $type_item->{items} }, { item => $key, props => $type->{structure}->{$key} };
		}
	    }
	    elsif ( $key_type eq "ARRAY" ) {
		push @{ $type_item->{items} }, { item => $key, props => { type => "array[string]", format => "array", description => "", required => 0, hidden => 0, transform => $key } };
	    }
	}
	push @{ $types }, $type_item;
    }

    $doc->{types} = $types;

    return $doc;
}

1;
