package Asterisk::API::Mapper::Generic;

use strict;
use warnings;


use JSON::XS;
use Types::Serialiser;
use Scalar::Util qw(looks_like_number);

our $VERSION = '1';

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    return $self;
}

sub prepare {
    my ( $self, $map ) = @_;

    my $mapping = { };

    foreach my $key ( keys % { $map } ) {
	my $key_type = ref $map->{$key}; # Types ( ref($arg) result ): "" - scalar, "HASH" - hash, "ARRAY" - array

	if ( $key_type eq "" ) {
	    if ( length $map->{$key} ) { # assigned transformation name
		$mapping->{'map'}->{$key} = $map->{$key};
		$mapping->{'reverse'}->{$map->{$key}} = $key;
	    }
	    else { # empty value assigned ( map 1:1 )
		$mapping->{'map'}->{$key} = $key;
		$mapping->{'reverse'}->{$key} = $key;
	    }

	    $mapping->{'type'}->{$key} = "string";
	}
	elsif ( $key_type eq "HASH" ) {
	    next if exists $map->{$key}->{hidden} && $map->{$key}->{hidden}; # skip field if assigned hidden flag

	    if ( exists $map->{$key}->{default} ) { # default value exists
		$mapping->{defaults}->{$key} = $map->{$key}->{default};
	    }

	    if ( exists $map->{$key}->{transform} && ref $map->{$key}->{transform} eq "ARRAY" ) { # transformation name array exists
		$mapping->{'map'}->{$key} = $map->{$key}->{transform};

		foreach my $transform_key ( @{ $map->{$key}->{transform} } ) {
		    $mapping->{'reverse'}->{$transform_key} = $key;
		}
	    }
	    elsif ( exists $map->{$key}->{transform} && length $map->{$key}->{transform} ) { # transformation name exists
		$mapping->{'map'}->{$key} = $map->{$key}->{transform};
		$mapping->{'reverse'}->{$map->{$key}->{transform}} = $key;
	    }
	    else { # transformation name does not exists
		$mapping->{'map'}->{$key} = $key;
		$mapping->{'reverse'}->{$key} = $key;
	    }

	    $mapping->{'type'}->{$key} = $map->{$key}->{type} || "string";

	}
	elsif ( $key_type eq "ARRAY" ) { # value is ARRAY ? ( map 1:1 )
		$mapping->{'map'}->{$key} = $key;
		$mapping->{'reverse'}->{$key} = $key;
	}
	else {
		$mapping->{'type'}->{$key} = "string";
	}

	$mapping->{'type'}->{$key} = "string" unless exists $mapping->{'type'}->{$key};
    }

    return $mapping;
}

sub fix_type {
    my ( $self, $out_hash, $types, $keys ) = @_;

    foreach my $key ( keys %{ $out_hash } ) {
        my $key_type = $types->{ $keys->{$key} };
        if ( $key_type =~ m/^int/i ) {
            $out_hash->{$key} = looks_like_number( $out_hash->{$key} ) ? eval { int $out_hash->{$key} } : $out_hash->{$key};
        }
	elsif ( $key_type =~ m/^bool/i ) {
            $out_hash->{$key} = $out_hash->{$key} ? Types::Serialiser::true : Types::Serialiser::false;
        }
        $out_hash->{$key} = '' unless defined $out_hash->{$key};
    }

    return $out_hash;
}

sub detect_type {
    my ( $self, $out_hash ) = @_;

    foreach my $key ( keys %{ $out_hash } ) {
	my $key_value = $out_hash->{$key};

	$key_value = $self->detect_type($key_value) if ref $key_value eq "HASH";

	unless ( ref $key_value ) {
	    if ( lc $key_value eq "true" ) {
		$key_value = Types::Serialiser::true;
	    }
	    elsif ( lc $key_value eq "false" ) {
		$key_value = Types::Serialiser::false;
	    }
	    elsif ( looks_like_number ( $key_value ) ) {
		$key_value = eval { int $out_hash->{$key} };
		$key_value = $out_hash->{$key} if $@;
	    }
	    elsif ( !defined $key_value ) {
		$key_value = '';
	    }
	}

	$out_hash->{$key} = $key_value;
    }

    return $out_hash;
}

1;
