package Asterisk::API::Structures::BulkExtensions;

use strict;
use warnings;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

our %out_hash_map = (
  "email_address" => {
    "description" => "Email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "ext" => {
    "description" => "Extension",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "transform" => "ext",
    "type" => "integer"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "firstname",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lastname",
    "type" => "string"
  },
  "phone_mac_address" => {
    "description" => "Phone MAC address",
    "format" => "6 octets",
    "hidden" => 0,
    "required" => 0,
    "transform" => "mac",
    "type" => "string"
  },
  "phone" => {
    "description" => "DID/Phone number",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "did",
    "type" => "string"
  },
  "secret" => {
    "description" => "Phone password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "secret",
    "type" => "string"
  },
  "pass" => {
    "description" => "WEB password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pass",
    "type" => "string"
  },
  "connect_enabled" => {
    "description" => "Thirdlane Connect enabled",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "connect_enabled",
    "type" => "boolean"
  }
);

my %in_hash_map = (
  "email_address" => {
    "description" => "Email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "ext" => {
    "description" => "Extension",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "transform" => "ext",
    "type" => "integer"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "firstname",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lastname",
    "type" => "string"
  },
  "phone_mac_address" => {
    "description" => "Phone MAC address",
    "format" => "6 octets",
    "hidden" => 0,
    "required" => 0,
    "transform" => "mac",
    "type" => "string"
  },
  "phone" => {
    "description" => "DID/Phone number",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "did",
    "type" => "string"
  },
  "secret" => {
    "description" => "Phone password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "secret",
    "type" => "string"
  },
  "pass" => {
    "description" => "WEB password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pass",
    "type" => "string"
  },
  "connect_enabled" => {
    "description" => "Thirdlane Connect enabled",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "connect_enabled",
    "type" => "boolean"
  }
);

our %requests = (
	types => [
	    {
		"type" => "extension",
		"structure" => \%out_hash_map
	    }
	],
	operations => [
	    {
		"path" => "/api/tenants/{tenantName}/extensions/",
		"method" => "POST",
		"summary" => "Add User Extension",
		"notes" => "Add User Extension",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "extension",
			"parameter" => "request",
			"sample" => q({ "ext" : 201, "last_name" : "Last", "first_name" : "First", "email_address" : "test@dot.net" })
		    },
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "extension"
			}
		]
	    }
	]
);

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}


sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{out} } ) { # fill default values
	    $out_hash->{$self->{map}->{out}->{$key}} = $self->{defaults}->{out}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{out}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{out}->{$key} && exists $hash->{$key};
    }

    $out_hash = $self->{mapper}->fix_type($out_hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $out_hash;
}


sub map_in {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{in} } ) { # fill default values
	    $out_hash->{$self->{map}->{in}->{$key}} = $self->{defaults}->{in}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{in}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{in}->{$key} && exists $hash->{$key};
    }

    return $out_hash;
}

1;
