package Asterisk::API::Structures::Conference;

use strict;
use warnings;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

our %out_hash_map = (
  "admin_password" => {
    "description" => "Admin password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "admin_password",
    "type" => "string"
  },
  "announce_count" => {
    "description" => "Announce count",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "announce_count",
    "type" => "string"
  },
  "announce_join" => {
    "description" => "Announce join",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "announce_join",
    "type" => "string"
  },
  "conference" => {
    "description" => "Conference Room number",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "sample" => 911,
    "transform" => "conference",
    "type" => "integer"
  },
  "description" => {
    "description" => "Description",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "description",
    "type" => "string"
  },
  "detect_talker" => {
    "description" => "Detect talker",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "detect_talker",
    "type" => "string"
  },
  "enable_moh" => {
    "description" => "Enable moh",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "enable_moh",
    "type" => "string"
  },
  "maxusers" => {
    "description" => "Maxusers",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxusers",
    "type" => "string"
  },
  "music" => {
    "description" => "Music",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "music",
    "type" => "string"
  },
  "password" => {
    "description" => "Password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "password",
    "type" => "string"
  },
  "present_menu" => {
    "description" => "Present menu",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "present_menu",
    "type" => "string"
  },
  "record" => {
    "description" => "Record",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record",
    "type" => "string"
  },
  "status" => {
    "description" => "Status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "status",
    "type" => "string"
  },
  "talker_optimization" => {
    "description" => "Talker optimization",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "talker_optimization",
    "type" => "string"
  },
  "users" => {
    "description" => "Users",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "users",
    "type" => "string"
  },
  "wait_marked" => {
    "description" => "Wait marked",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "wait_marked",
    "type" => "string"
  }

);

my %in_hash_map = (
  "admin_password" => {
    "description" => "Admin password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "admin_password",
    "type" => "string"
  },
  "announce_count" => {
    "description" => "Announce count",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "announce_count",
    "type" => "string"
  },
  "announce_join" => {
    "description" => "Announce join",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "announce_join",
    "type" => "string"
  },
  "conference" => {
    "description" => "Conference Room number",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "sample" => 911,
    "transform" => "conference",
    "type" => "integer"
  },
  "description" => {
    "description" => "Description",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "description",
    "type" => "string"
  },
  "detect_talker" => {
    "description" => "Detect talker",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "detect_talker",
    "type" => "string"
  },
  "enable_moh" => {
    "description" => "Enable moh",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "enable_moh",
    "type" => "string"
  },
  "maxusers" => {
    "description" => "Maxusers",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxusers",
    "type" => "string"
  },
  "music" => {
    "description" => "Music",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "music",
    "type" => "string"
  },
  "password" => {
    "description" => "Password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "password",
    "type" => "string"
  },
  "present_menu" => {
    "description" => "Present menu",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "present_menu",
    "type" => "string"
  },
  "record" => {
    "description" => "Record",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record",
    "type" => "string"
  },
  "status" => {
    "description" => "Status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "status",
    "type" => "string"
  },
  "talker_optimization" => {
    "description" => "Talker optimization",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "talker_optimization",
    "type" => "string"
  },
  "users" => {
    "description" => "Users",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "users",
    "type" => "string"
  },
  "wait_marked" => {
    "description" => "Wait marked",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "wait_marked",
    "type" => "string"
  }

);

our %requests = (
	types => [
	    {
		"type" => "conference",
		"structure" => \%out_hash_map
	    }
	],
	operations => [
	    {
		"path" => "/api/conference/{tenantName}/",
		"method" => "GET",
		"summary" => "Get all Conferences",
		"notes" => "Get all Conferences",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "conference"
			}
		]
	    },
	    {
		"path" => "/api/conference/{tenantName}/{conference}",
		"method" => "GET",
		"summary" => "Get Single Conference",
		"notes" => "Get Single Conference Room",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    },
		    {
			"name" => "conference",
			"description" => "Conference",
			"required" => 0,
			"type" => "integer",
			"parameter" => "path"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "conference"
			}
		]
	    },
	    {
		"path" => "/api/conference/{tenantName}/",
		"method" => "POST",
		"summary" => "Add Conference Room",
		"notes" => "Add Conference Room",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "conference",
			"parameter" => "request",
			"sample" => q({"conference" : "991", "enable_moh" : "1", "status" : "true", "maxusers" : "20", "talker_optimization" : "true", "present_menu" : "0", "announce_join" : "1", "record" : "0", "announce_count" : "true", "wait_marked" : "0", "music" : "default", "detect_talker" : "true", "users" : "thirdlane-9000,thirdlane-9009", "description" : "emergency room", "password" : "123", "admin_password" : "111"})
		    },
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "conference"
			}
		]
	    },
	    {
		"path" => "/api/conference/{tenantName}/{conference}",
		"method" => "PUT",
		"summary" => "Update Conference Room",
		"notes" => "Update one Conference Room",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "conference",
			"parameter" => "request",
			"sample" => q({"conference" : "991", "enable_moh" : "1", "status" : "true", "maxusers" : "20", "talker_optimization" : "true", "present_menu" : "0", "announce_join" : "1", "record" : "0", "announce_count" : "true", "wait_marked" : "0", "music" : "default", "detect_talker" : "true", "users" : "thirdlane-9000,thirdlane-9009", "description" : "emergency room", "password" : "123", "admin_password" : "111"})
		    },
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    },
		    {
			"name" => "conference",
			"description" => "Conference",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "201"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "conference"
			}
		]
	    },
	    {
		"path" => "/api/conference/{tenantName}/{conference}",
		"method" => "DELETE",
		"summary" => "Delete Conference Room",
		"notes" => "Delete one Conference Room",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    },
		    {
			"name" => "conference",
			"description" => "Conference",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "201"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "conference"
			}
		]
	    }
	]
);

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}


sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{out} } ) { # fill default values
	    $out_hash->{$self->{map}->{out}->{$key}} = $self->{defaults}->{out}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{out}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{out}->{$key} && exists $hash->{$key};
    }

    $out_hash = $self->{mapper}->fix_type($out_hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $out_hash;
}


sub map_in {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{in} } ) { # fill default values
	    $out_hash->{$self->{map}->{in}->{$key}} = $self->{defaults}->{in}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{in}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{in}->{$key} && exists $hash->{$key};
    }

    return $out_hash;
}

1;
