package Asterisk::API::Structures::ContactsPersonal;

use strict;
use warnings;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

our %out_hash_map = (
  "alias" => {
    "description" => "User alias",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "alias",
    "type" => "string"
  },
  "company" => {
    "description" => "Company",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "company",
    "type" => "string"
  },
  "contact_source" => {
    "description" => "Contact Source",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "vtiger",
    "transform" => "contact_source",
    "type" => "string"
  },
  "contact_source_contact_id" => {
    "description" => "Contact Source Record ID",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "00x1",
    "transform" => "contact_source_id",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "email" => {
    "description" => "Email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "firstname" => {
    "description" => "Firstname",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "first_name",
    "type" => "string"
  },
  "id" => {
    "description" => "Record ID",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "sample" => 1,
    "transform" => "id",
    "type" => "integer"
  },
  "image_url" => {
    "description" => "Image url",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "image_url",
    "type" => "string"
  },
  "jid" => {
    "description" => "Jid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "jid",
    "type" => "string"
  },
  "lastname" => {
    "description" => "Lastname",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "last_name",
    "type" => "string"
  },
  "note" => {
    "description" => "Note",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "note",
    "type" => "string"
  },
  "phone_numbers" => {
    "description" => "Phone numbers",
    "format" => "array",
    "hidden" => 0,
    "required" => 0,
    "sample" => " [ { \"type\" : \"extension\", \"value\" : \"1001\" } ] ",
    "transform" => "phone_numbers",
    "type" => "phone_number"
  },
  "updated" => {
    "description" => "Updated",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "updated",
    "type" => "string"
  },
  "updated_epoch" => {
    "description" => "Updated UTC epoch value",
    "format" => "unixtime",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "updated_epoch",
    "type" => "integer"
  },
  "url" => {
    "description" => "External URL",
    "format" => "url",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "url",
    "type" => "string"
  },
  "phone_numbers" => {
    "description" => "Phone numbers",
    "format" => "array",
    "hidden" => 0,
    "required" => 0,
    "sample" => " [ { \"type\" : \"extension\", \"value\" : \"1001\" } ] ",
    "transform" => "phone_numbers",
    "type" => "phone_number"
  }

);

my %in_hash_map = (
  "alias" => {
    "description" => "User alias",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "alias",
    "type" => "string"
  },
  "company" => {
    "description" => "Company",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "company",
    "type" => "string"
  },
  "contact_source" => {
    "description" => "Contact Source",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "vtiger",
    "transform" => "contact_source",
    "type" => "string"
  },
  "contact_source_id" => {
    "description" => "Contact Source Record ID",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "00x1",
    "transform" => "contact_source_contact_id",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "email" => {
    "description" => "Email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "ext" => {
    "description" => "Ext",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext",
    "type" => "string"
  },
  "ext_speeddial" => {
    "description" => "Ext speeddial",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext_speeddial",
    "type" => "string"
  },
  "ext_type" => {
    "description" => "Ext type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext_type",
    "type" => "string"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "firstname",
    "type" => "string"
  },
  "home" => {
    "description" => "Home",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "home",
    "type" => "string"
  },
  "home_speeddial" => {
    "description" => "Home speeddial",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "home_speeddial",
    "type" => "string"
  },
  "home_type" => {
    "description" => "Home type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "home_type",
    "type" => "string"
  },
  "id" => {
    "description" => "Record ID",
    "format" => "int64",
    "hidden" => 1,
    "required" => 0,
    "sample" => 1,
    "transform" => "id",
    "type" => "integer"
  },
  "image_url" => {
    "description" => "Image url",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "image_url",
    "type" => "string"
  },
  "jid" => {
    "description" => "Jid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "jid",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lastname",
    "type" => "string"
  },
  "mobile" => {
    "description" => "Mobile",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile",
    "type" => "string"
  },
  "mobile_speeddial" => {
    "description" => "Mobile speeddial",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile_speeddial",
    "type" => "string"
  },
  "mobile_type" => {
    "description" => "Mobile type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile_type",
    "type" => "string"
  },
  "note" => {
    "description" => "Note",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "note",
    "type" => "string"
  },
  "office" => {
    "description" => "Office",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "office",
    "type" => "string"
  },
  "office_speeddial" => {
    "description" => "Office speeddial",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "office_speeddial",
    "type" => "string"
  },
  "office_type" => {
    "description" => "Office type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "office_type",
    "type" => "string"
  },
  "other" => {
    "description" => "Other",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other",
    "type" => "string"
  },
  "other_speeddial" => {
    "description" => "Other speeddial",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other_speeddial",
    "type" => "string"
  },
  "other_type" => {
    "description" => "Other type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other_type",
    "type" => "string"
  },
  "phone_numbers" => {
    "description" => "Phone numbers",
    "format" => "array",
    "hidden" => 0,
    "required" => 0,
    "sample" => " [ { \"type\" : \"extension\", \"value\" : \"1001\" } ] ",
    "transform" => "phone_numbers",
    "type" => "phone_number"
  }

);

our %phone_number_type = (
	type => {
	    type => "string",
	    format => "one of [extension|office|home|mobile|other]",
	    description => "Phone number type",
	    required => 1,
	    hidden => 0,
	    transform => "type",
	    sample => q(extension)
	},
	value => {
	    type => "string",
	    format => "free",
	    description => "Phone number",
	    required => 1,
	    hidden => 0,
	    transform => "value",
	    sample => q(10001)
	}
);

our %requests = (
	types => [
	    {
		"type" => "contact_personal",
		"structure" => \%out_hash_map
	    },
	    {
		"type" => "phone_number",
		"structure" => \%phone_number_type
	    }
	],
	operations => [
	    {
		"path" => "/api/user/contacts/personal/",
		"method" => "GET",
		"summary" => "Get User's Personal Contacts",
		"notes" => "Get User's Personal Contacts",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "contact_personal"
			}
		]
	    },
	    {
		"path" => "/api/user/contacts/personal/{contactId}",
		"method" => "GET",
		"summary" => "Get one Personal Contact",
		"notes" => "Get one Personal Contact",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "contactId",
			"description" => "Contact ID",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "1"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "contact_personal"
			}
		]
	    },
	    {
		"path" => "/api/user/contacts/personal/",
		"method" => "POST",
		"summary" => "Add one Personal Contact",
		"notes" => "Add one Personal Contact",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "contact_personal",
			"parameter" => "request",
			"sample" => q({ "last_name" : "Last", "first_name" : "First", "phone_numbers" : [ { "type" : "extension", "value" : "201" } ] })
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "contact_personal"
			}
		]
	    },
	    {
		"path" => "/api/user/contacts/personal/{contactId}",
		"method" => "PUT",
		"summary" => "Update Personal Contact",
		"notes" => "Update Personal Contact",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "contact_personal",
			"parameter" => "request",
			"sample" => q({ "last_name" : "New Last", "phone_numbers" : [ { "type" : "extension", "value" : "201" } ] })
		    },
		    {
			"name" => "contactId",
			"description" => "Contact ID",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "1"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "contact_personal"
			}
		]
	    },
	    {
		"path" => "/api/user/contacts/personal/{contactId}",
		"method" => "DELETE",
		"summary" => "Delete Personal Contact",
		"notes" => "Delete Personal Contact",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "contactId",
			"description" => "Contact ID",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "1"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "contact_personal"
			}
		]
	    }
	]
);


sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}


sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{out} } ) { # fill default values
	    $out_hash->{$self->{map}->{out}->{$key}} = $self->{defaults}->{out}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{out}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{out}->{$key} && exists $hash->{$key};
    }

    $out_hash = $self->{mapper}->fix_type($out_hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $out_hash;
}

sub map_in {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{in} } ) { # fill default values
	    $out_hash->{$self->{map}->{in}->{$key}} = $self->{defaults}->{in}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{in}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{in}->{$key} && exists $hash->{$key};
    }

    return $out_hash;
}

1;
