package Asterisk::API::Structures::QueryCRM;

use strict;
use warnings;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

use DateTime;
use DateTime::Format::Strptime;

our %out_hash_map = (
  "company" => {
    "description" => "Company",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "company",
    "type" => "string"
  },
  "contact_source" => {
    "description" => "Contact Source",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "vtiger",
    "transform" => "contact_source",
    "type" => "string"
  },
  "contact_source_contact_id" => {
    "description" => "Contact Source Record ID",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "00x1",
    "transform" => "contact_source_id",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "email" => {
    "description" => "Email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "firstname" => {
    "description" => "Firstname",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "first_name",
    "type" => "string"
  },
  "id" => {
    "description" => "Record ID",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "sample" => 1,
    "transform" => "id",
    "type" => "integer"
  },
  "lastname" => {
    "description" => "Lastname",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "last_name",
    "type" => "string"
  },
  "note" => {
    "description" => "Note",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "note",
    "type" => "string"
  },
  "phone_numbers" => {
    "description" => "Phone numbers",
    "format" => "array",
    "hidden" => 0,
    "required" => 0,
    "sample" => " [ { \"type\" : \"extension\", \"value\" : \"1001\" } ] ",
    "transform" => "phone_numbers",
    "type" => "phone_number"
  }

);

my %in_hash_map = (
);

our %requests = (
	types => [
	    {
		"type" => "query_crm",
		"structure" => \%out_hash_map
	    }
	],
	operations => [
	    {
		"path" => "/api/query/crm/{key}",
		"method" => "GET",
		"summary" => "Get CRM Records",
		"notes" => "Get CRM Records List",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "key",
			"description" => "Query Key",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "Allan"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "query_crm"
			}
		]
	    }
	]
);

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}

sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ($self, $hash) = @_;

    $hash = $self->{mapper}->fix_type($hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $hash;
}

sub map_in {
    my ($self, $hash) = @_;

    return $hash;
}

1;
