package Asterisk::API::Structures::SingleExtensions;

use strict;
use warnings;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

our %out_hash_map = (
  "RECORD" => {
    "description" => "RECORD",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "recording_status",
    "type" => "string"
  },
  "RECORD_ALL" => {
    "description" => "RECORD ALL",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_all",
    "type" => "string"
  },
  "RECORD_EXT_IN" => {
    "description" => "RECORD EXT IN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_ext_in",
    "type" => "string"
  },
  "RECORD_EXT_OUT" => {
    "description" => "RECORD EXT OUT",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_ext_out",
    "type" => "string"
  },
  "RECORD_INT_IN" => {
    "description" => "RECORD INT IN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_int_in",
    "type" => "string"
  },
  "RECORD_INT_OUT" => {
    "description" => "RECORD INT OUT",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_int_out",
    "type" => "string"
  },
  "RECORD_ONE_TOUCH" => {
    "description" => "RECORD ONE TOUCH",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_one_touch",
    "type" => "string"
  },
  "accountcode" => {
    "description" => "Accountcode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "accountcode",
    "type" => "string"
  },
  "allowrecording" => {
    "description" => "Allowrecording",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "allowrecording",
    "type" => "string"
  },
  "call_blocking" => {
    "description" => "Call blocking",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_blocking",
    "type" => "string"
  },
  "call_forwarding" => {
    "description" => "Call forwarding",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_forwarding",
    "type" => "string"
  },
  "call_history" => {
    "description" => "Call history",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_history",
    "type" => "string"
  },
  "call_screening" => {
    "description" => "Call screening",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_screening",
    "type" => "string"
  },
  "callerid" => {
    "description" => "Callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid",
    "type" => "string"
  },
  "callerid_name_option" => {
    "description" => "Callerid name option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_name_option",
    "type" => "string"
  },
  "callerid_option" => {
    "description" => "Callerid option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_option",
    "type" => "string"
  },
  "callerid_override" => {
    "description" => "Callerid override",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_override",
    "type" => "string"
  },
  "contacts" => {
    "description" => "Contacts",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "contacts",
    "type" => "string"
  },
  "create_conference" => {
    "description" => "Create conference",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_conference",
    "type" => "string"
  },
  "create_vm" => {
    "description" => "Create vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_vm",
    "type" => "string"
  },
  "crm" => {
    "description" => "Crm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm",
    "type" => "string"
  },
  "crm_cidminlength" => {
    "description" => "Crm cidminlength",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidminlength",
    "type" => "string"
  },
  "crm_cidstrip" => {
    "description" => "Crm cidstrip",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidstrip",
    "type" => "string"
  },
  "crm_url_format" => {
    "description" => "Crm url format",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_url_format",
    "type" => "string"
  },
  "crm_use_defaults" => {
    "description" => "Crm use defaults",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_use_defaults",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "dialbyname_option" => {
    "description" => "Dialbyname option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "dialbyname_option",
    "type" => "string"
  },
  "dialbyname_override" => {
    "description" => "Dialbyname override",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "dialbyname_override",
    "type" => "string"
  },
  "did" => {
    "description" => "Did",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "did",
    "type" => "string"
  },
  "email" => {
    "description" => "Email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email_address",
    "type" => "string"
  },
  "emergency_callerid" => {
    "description" => "Emergency callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "emergency_callerid",
    "type" => "string"
  },
  "enduser_company_contacts" => {
    "description" => "Enduser company contacts",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "enduser_company_contacts",
    "type" => "string"
  },
  "ext_callerid" => {
    "description" => "Ext callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext_callerid",
    "type" => "string"
  },
  "ext_callerid_name" => {
    "description" => "Ext callerid name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext_callerid_name",
    "type" => "string"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "first_name",
    "type" => "string"
  },
  "forwardattach" => {
    "description" => "Forwardattach",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardattach",
    "type" => "string"
  },
  "forwarddelete" => {
    "description" => "Forwarddelete",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarddelete",
    "type" => "string"
  },
  "forwardemail" => {
    "description" => "Forwardemail",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardemail",
    "type" => "string"
  },
  "forwardpager" => {
    "description" => "Forwardpager",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardpager",
    "type" => "string"
  },
  "id" => {
    "description" => "Id",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "id",
    "type" => "string"
  },
  "lang" => {
    "description" => "Lang",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lang",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "last_name",
    "type" => "string"
  },
  "mac" => {
    "description" => "Phone MAC address",
    "format" => "6 octets",
    "hidden" => 0,
    "required" => 0,
    "sample" => "D3:67:41:12:AE:5B",
    "transform" => "phone_mac_address",
    "type" => "string"
  },
  "mailbox" => {
    "description" => "Mailbox",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mailbox",
    "type" => "string"
  },
  "mobile" => {
    "description" => "Mobile",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile",
    "type" => "string"
  },
  "model" => {
    "description" => "Model",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "model",
    "type" => "string"
  },
  "name" => {
    "description" => "Name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "name",
    "type" => "string"
  },
  "pager" => {
    "description" => "Pager",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pager",
    "type" => "string"
  },
  "pass" => {
    "description" => "WEB password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pass",
    "type" => "string"
  },
  "phones" => {
    "description" => "Phones",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "phones",
    "type" => "string"
  },
  "pin" => {
    "description" => "Pin",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pin",
    "type" => "string"
  },
  "preferences" => {
    "description" => "Preferences",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "preferences",
    "type" => "string"
  },
  "protocol" => {
    "description" => "Protocol",
    "default" => "SIP",
    "format" => "raw",
    "hidden" => 0,
    "required" => 1,
    "sample" => "",
    "transform" => "protocol",
    "type" => "string"
  },
  "recording_listen" => {
    "description" => "Recording listen",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "recording_listen",
    "type" => "string"
  },
  "secret" => {
    "description" => "Phone password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "secret",
    "type" => "string"
  },
  "send_faxes" => {
    "description" => "Send faxes",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "send_faxes",
    "type" => "string"
  },
  "set_musiconhold" => {
    "description" => "Set musiconhold",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "set_musiconhold",
    "type" => "string"
  },
  "set_officemode" => {
    "description" => "Set officemode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "set_officemode",
    "type" => "string"
  },
  "tag" => {
    "description" => "Tag",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tag",
    "type" => "string"
  },
  "uname" => {
    "description" => "Extension",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "sample" => 201,
    "transform" => "ext",
    "type" => "integer"
  },
  "user_portal" => {
    "description" => "User portal",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "user_portal",
    "type" => "string"
  },
  "vm" => {
    "description" => "Vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "vm",
    "type" => "string"
  },
  "vm_settings" => {
    "description" => "Vm settings",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "vm_settings",
    "type" => "string"
  },
  "device_type" => {
    "description" => "Device type (0 - SIP; 1 - WebRTC)",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "0",
    "transform" => "device_type",
    "type" => "integer"
  },
  "connect_enabled" => {
    "description" => "Thirdlane Connect enabled",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "0",
    "transform" => "connect_enabled",
    "type" => "boolean"
  }

);

my %in_hash_map = (
  "accountcode" => {
    "description" => "Accountcode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "accountcode",
    "type" => "string"
  },
  "allowrecording" => {
    "description" => "Allowrecording",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "allowrecording",
    "type" => "string"
  },
  "call_blocking" => {
    "description" => "Call blocking",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_blocking",
    "type" => "string"
  },
  "call_forwarding" => {
    "description" => "Call forwarding",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_forwarding",
    "type" => "string"
  },
  "call_history" => {
    "description" => "Call history",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_history",
    "type" => "string"
  },
  "call_screening" => {
    "description" => "Call screening",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call_screening",
    "type" => "string"
  },
  "callerid" => {
    "description" => "Callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid",
    "type" => "string"
  },
  "callerid_name_option" => {
    "description" => "Callerid name option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_name_option",
    "type" => "string"
  },
  "callerid_option" => {
    "description" => "Callerid option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_option",
    "type" => "string"
  },
  "callerid_override" => {
    "description" => "Callerid override",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_override",
    "type" => "string"
  },
  "contacts" => {
    "description" => "Contacts",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "contacts",
    "type" => "string"
  },
  "create_conference" => {
    "description" => "Create conference",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_conference",
    "type" => "string"
  },
  "create_vm" => {
    "description" => "Create vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_vm",
    "type" => "string"
  },
  "crm" => {
    "description" => "Crm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm",
    "type" => "string"
  },
  "crm_cidminlength" => {
    "description" => "Crm cidminlength",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidminlength",
    "type" => "string"
  },
  "crm_cidstrip" => {
    "description" => "Crm cidstrip",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidstrip",
    "type" => "string"
  },
  "crm_url_format" => {
    "description" => "Crm url format",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_url_format",
    "type" => "string"
  },
  "crm_use_defaults" => {
    "description" => "Crm use defaults",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_use_defaults",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "dialbyname_option" => {
    "description" => "Dialbyname option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "dialbyname_option",
    "type" => "string"
  },
  "dialbyname_override" => {
    "description" => "Dialbyname override",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "dialbyname_override",
    "type" => "string"
  },
  "did" => {
    "description" => "Did",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "did",
    "type" => "string"
  },
  "email_address" => {
    "description" => "Email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "emergency_callerid" => {
    "description" => "Emergency callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "emergency_callerid",
    "type" => "string"
  },
  "enduser_company_contacts" => {
    "description" => "Enduser company contacts",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "enduser_company_contacts",
    "type" => "string"
  },
  "ext" => {
    "description" => "Extension",
    "format" => "int64",
    "hidden" => 0,
    "required" => 1,
    "transform" => "uname",
    "type" => "integer"
  },
  "ext_callerid" => {
    "description" => "Ext callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext_callerid",
    "type" => "string"
  },
  "ext_callerid_name" => {
    "description" => "Ext callerid name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "ext_callerid_name",
    "type" => "string"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "first_name",
    "type" => "string"
  },
  "forwardattach" => {
    "description" => "Forwardattach",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardattach",
    "type" => "string"
  },
  "forwarddelete" => {
    "description" => "Forwarddelete",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarddelete",
    "type" => "string"
  },
  "forwardemail" => {
    "description" => "Forwardemail",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardemail",
    "type" => "string"
  },
  "forwardpager" => {
    "description" => "Forwardpager",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardpager",
    "type" => "string"
  },
  "id" => {
    "description" => "Extensions internal ID",
    "format" => "64bits",
    "hidden" => 1,
    "required" => 0,
    "transform" => "id",
    "type" => "integer"
  },
  "lang" => {
    "description" => "Lang",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lang",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "last_name",
    "type" => "string"
  },
  "mailbox" => {
    "description" => "Mailbox",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mailbox",
    "type" => "string"
  },
  "mobile" => {
    "description" => "Mobile",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile",
    "type" => "string"
  },
  "model" => {
    "description" => "Model",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "model",
    "type" => "string"
  },
  "name" => {
    "description" => "Name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "name",
    "type" => "string"
  },
  "pager" => {
    "description" => "Pager",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pager",
    "type" => "string"
  },
  "pass" => {
    "description" => "WEB password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pass",
    "type" => "string"
  },
  "phone_mac_address" => {
    "description" => "Phone MAC address",
    "format" => "6 octets",
    "hidden" => 0,
    "required" => 1,
    "transform" => "mac",
    "type" => "string"
  },
  "phones" => {
    "default" => "equal",
    "description" => "Phones",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "equal",
    "transform" => "phones",
    "type" => "string"
  },
  "pin" => {
    "description" => "Pin",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pin",
    "type" => "string"
  },
  "preferences" => {
    "description" => "Preferences",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "preferences",
    "type" => "string"
  },
  "protocol" => {
    "description" => "Protocol",
    "default" => "SIP",
    "format" => "raw",
    "hidden" => 0,
    "required" => 1,
    "sample" => "",
    "transform" => "protocol",
    "type" => "string"
  },
  "record_all" => {
    "description" => "Record all",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_ALL",
    "type" => "string"
  },
  "record_ext_in" => {
    "description" => "Record ext in",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_EXT_IN",
    "type" => "string"
  },
  "record_ext_out" => {
    "description" => "Record ext out",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_EXT_OUT",
    "type" => "string"
  },
  "record_int_in" => {
    "description" => "Record int in",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_INT_IN",
    "type" => "string"
  },
  "record_int_out" => {
    "description" => "Record int out",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_INT_OUT",
    "type" => "string"
  },
  "record_one_touch" => {
    "description" => "Record one touch",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_ONE_TOUCH",
    "type" => "string"
  },
  "recording_listen" => {
    "description" => "Recording listen",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "recording_listen",
    "type" => "string"
  },
  "secret" => {
    "description" => "Phone password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "secret",
    "type" => "string"
  },
  "send_faxes" => {
    "description" => "Send faxes",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "send_faxes",
    "type" => "string"
  },
  "set_musiconhold" => {
    "description" => "Set musiconhold",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "set_musiconhold",
    "type" => "string"
  },
  "set_officemode" => {
    "description" => "Set officemode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "set_officemode",
    "type" => "string"
  },
  "tag" => {
    "description" => "Tag",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tag",
    "type" => "string"
  },
  "user_portal" => {
    "description" => "User portal",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "user_portal",
    "type" => "string"
  },
  "vm" => {
    "description" => "Vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "vm",
    "type" => "string"
  },
  "vm_settings" => {
    "description" => "Vm settings",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "vm_settings",
    "type" => "string"
  },
  "device_type" => {
    "description" => "Device type (0 - SIP; 1 - WebRTC)",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "0",
    "transform" => "device_type",
    "type" => "integer"
  },
  "connect_enabled" => {
    "description" => "Thirdlane Connect enabled",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "0",
    "transform" => "connect_enabled",
    "type" => "boolean"
  }

);

our %requests = (
	types => [
	    {
		"type" => "extension",
		"structure" => \%out_hash_map
	    }
	],
	operations => [
	    {
		"path" => "/api/extensions/",
		"method" => "GET",
		"summary" => "Get all User Extensions",
		"notes" => "Get all User extensions",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "extension"
			}
		]
	    },
	    {
		"path" => "/api/extensions/{extension}",
		"method" => "GET",
		"summary" => "Get one User Extension",
		"notes" => "Get one User Extension",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "extension",
			"description" => "Extension",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "extension"
			}
		]
	    },
	    {
		"path" => "/api/extensions/",
		"method" => "POST",
		"summary" => "Add User Extension",
		"notes" => "Add one User Extension",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "extension",
			"parameter" => "request",
			"sample" => q({"ext" : "201" , "first_name" : "First" , "last_name" : "Last" , "email_address" : "test@email.net",  "protocol" : "SIP" })
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "extension"
			}
		]
	    },
	    {
		"path" => "/api/extensions/{extension}",
		"method" => "PUT",
		"summary" => "Update User Extension",
		"notes" => "Update one User Extension",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "extension",
			"parameter" => "request",
			"sample" => q({ "first_name" : "New First" , "last_name" : "New Last" , "email_address" : "newemail@dot.net" })
		    },
		    {
			"name" => "extension",
			"description" => "Extension",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "201"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "extension"
			}
		]
	    },
	    {
		"path" => "/api/extensions/{extension}",
		"method" => "DELETE",
		"summary" => "Delete User Extension",
		"notes" => "Delete one User Extension",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "extension",
			"description" => "Extension",
			"required" => 1,
			"type" => "integer",
			"parameter" => "path",
			"sample" => "201"
		    }
			],
		"responses" => [
			{
				"code" => 200,
				"type" => "extension"
			}
		]
	    }
	]
);

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}


sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{out} } ) { # fill default values
	    $out_hash->{$self->{map}->{out}->{$key}} = $self->{defaults}->{out}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{out}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{out}->{$key} && exists $hash->{$key};
    }

    $out_hash = $self->{mapper}->fix_type($out_hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $out_hash;
}

sub map_in {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{in} } ) { # fill default values
	    $out_hash->{$self->{map}->{in}->{$key}} = $self->{defaults}->{in}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{in}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{in}->{$key} && exists $hash->{$key};
    }

    return $out_hash;
}

1;
