package Asterisk::API::Structures::Tenants;

use strict;
use warnings;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

our %out_hash_map = (
  "RECORD" => {
    "description" => "RECORD",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record",
    "type" => "string"
  },
  "RECORD_ALL" => {
    "description" => "RECORD ALL",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_all",
    "type" => "string"
  },
  "RECORD_EXT_IN" => {
    "description" => "RECORD EXT IN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_ext_in",
    "type" => "string"
  },
  "RECORD_EXT_OUT" => {
    "description" => "RECORD EXT OUT",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_ext_out",
    "type" => "string"
  },
  "RECORD_INT_IN" => {
    "description" => "RECORD INT IN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_int_in",
    "type" => "string"
  },
  "RECORD_INT_OUT" => {
    "description" => "RECORD INT OUT",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_int_out",
    "type" => "string"
  },
  "RECORD_ONE_TOUCH" => {
    "description" => "RECORD ONE TOUCH",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_one_touch",
    "type" => "string"
  },
  "accountcode" => {
    "description" => "Accountcode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "accountcode",
    "type" => "string"
  },
  "allowaccountcode" => {
    "description" => "Allowaccountcode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "allowaccountcode",
    "type" => "string"
  },
  "allowcallerid" => {
    "description" => "Allowcallerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "allowcallerid",
    "type" => "string"
  },
  "attach" => {
    "description" => "Attach",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "attach",
    "type" => "string"
  },
  "call-limit" => {
    "description" => "Call-limit",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call-limit",
    "type" => "string"
  },
  "callerid" => {
    "description" => "Callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid",
    "type" => "string"
  },
  "callerid_name" => {
    "description" => "Callerid name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_name",
    "type" => "string"
  },
  "canreinvite" => {
    "description" => "Canreinvite",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "canreinvite",
    "type" => "string"
  },
  "codec" => {
    "description" => "Codec",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "codec",
    "type" => "string"
  },
  "conf_announce_count" => {
    "description" => "Conf announce count",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_announce_count",
    "type" => "string"
  },
  "conf_announce_join" => {
    "description" => "Conf announce join",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_announce_join",
    "type" => "string"
  },
  "conf_detect_talker" => {
    "description" => "Conf detect talker",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_detect_talker",
    "type" => "string"
  },
  "conf_enable_moh" => {
    "description" => "Conf enable moh",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_enable_moh",
    "type" => "string"
  },
  "conf_maxusers" => {
    "description" => "Conf maxusers",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_maxusers",
    "type" => "string"
  },
  "conf_music" => {
    "description" => "Conf music",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_music",
    "type" => "string"
  },
  "conf_present_menu" => {
    "description" => "Conf present menu",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_present_menu",
    "type" => "string"
  },
  "conf_record" => {
    "description" => "Conf record",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_record",
    "type" => "string"
  },
  "conf_talker_optimization" => {
    "description" => "Conf talker optimization",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_talker_optimization",
    "type" => "string"
  },
  "conf_wait_marked" => {
    "description" => "Conf wait marked",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_wait_marked",
    "type" => "string"
  },
  "contact_center" => {
    "description" => "Contact center",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "contact_center",
    "type" => "string"
  },
  "context" => {
    "description" => "Context",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "context",
    "type" => "string"
  },
  "create_conference" => {
    "description" => "Create conference",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_conference",
    "type" => "string"
  },
  "create_vm" => {
    "description" => "Create vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_vm",
    "type" => "string"
  },
  "crm_cidminlength" => {
    "description" => "Crm cidminlength",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidminlength",
    "type" => "string"
  },
  "crm_cidstrip" => {
    "description" => "Crm cidstrip",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidstrip",
    "type" => "string"
  },
  "crm_url_format" => {
    "description" => "Crm url format",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_url_format",
    "type" => "string"
  },
  "crm_use_defaults" => {
    "description" => "Crm use defaults",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_use_defaults",
    "type" => "string"
  },
  "default_mailbox_pin" => {
    "description" => "Default mailbox pin",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "default_mailbox_pin",
    "type" => "string"
  },
  "default_web_password" => {
    "description" => "Default web password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "default_web_password",
    "type" => "string"
  },
  "delete" => {
    "description" => "Delete",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "delete",
    "type" => "string"
  },
  "dtmfmode" => {
    "description" => "Dtmfmode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "dtmfmode",
    "type" => "string"
  },
  "emergency_callerid" => {
    "description" => "Emergency callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "emergency_callerid",
    "type" => "string"
  },
  "entered_other" => {
    "description" => "Entered other",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "entered_other",
    "type" => "string"
  },
  "entered_other_vm" => {
    "description" => "Entered other vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "entered_other_vm",
    "type" => "string"
  },
  "fax_default_paper_size" => {
    "description" => "Fax default paper size",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_default_paper_size",
    "type" => "string"
  },
  "fax_email_address" => {
    "description" => "Fax email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_email_address",
    "type" => "string"
  },
  "fax_maximum_number_of_retries" => {
    "description" => "Fax maximum number of retries",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_maximum_number_of_retries",
    "type" => "string"
  },
  "fax_notify_on_failure_and_success" => {
    "description" => "Fax notify on failure and success",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_notify_on_failure_and_success",
    "type" => "string"
  },
  "fax_retry_time" => {
    "description" => "Fax retry time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_retry_time",
    "type" => "string"
  },
  "fax_settings_station_id" => {
    "description" => "Fax settings station id",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_settings_station_id",
    "type" => "string"
  },
  "fax_wait_time" => {
    "description" => "Fax wait time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_wait_time",
    "type" => "string"
  },
  "forwardemail" => {
    "description" => "Forwardemail",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardemail",
    "type" => "string"
  },
  "id" => {
    "description" => "Id",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "id",
    "type" => "string"
  },
  "intalertinfo" => {
    "description" => "Intalertinfo",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "intalertinfo",
    "type" => "string"
  },
  "lang" => {
    "description" => "Lang",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lang",
    "type" => "string"
  },
  "mailbox_pin_length" => {
    "description" => "Mailbox pin length",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mailbox_pin_length",
    "type" => "string"
  },
  "mailbox_pin_option" => {
    "description" => "Mailbox pin option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mailbox_pin_option",
    "type" => "string"
  },
  "mappend" => {
    "description" => "Mappend",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mappend",
    "type" => "string"
  },
  "maxagents" => {
    "description" => "Maxagents",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxagents",
    "type" => "string"
  },
  "maxcalls" => {
    "description" => "Maxcalls",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxcalls",
    "type" => "string"
  },
  "maxconferences" => {
    "description" => "Maxconferences",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxconferences",
    "type" => "string"
  },
  "maxdevices" => {
    "description" => "Maxdevices",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxdevices",
    "type" => "string"
  },
  "maxmailboxes" => {
    "description" => "Maxmailboxes",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxmailboxes",
    "type" => "string"
  },
  "maxphones" => {
    "description" => "Maxphones",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxphones",
    "type" => "string"
  },
  "maxqueues" => {
    "description" => "Maxqueues",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxqueues",
    "type" => "string"
  },
  "maxusers" => {
    "description" => "Maxusers",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxusers",
    "type" => "string"
  },
  "model" => {
    "description" => "Model",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "model",
    "type" => "string"
  },
  "musiconhold" => {
    "description" => "Musiconhold",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "musiconhold",
    "type" => "string"
  },
  "name" => {
    "description" => "Name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "description",
    "type" => "string"
  },
  "nat" => {
    "description" => "Nat",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "nat",
    "type" => "string"
  },
  "officemode" => {
    "description" => "Officemode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "officemode",
    "type" => "string"
  },
  "other" => {
    "description" => "Other",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other",
    "type" => "string"
  },
  "panel" => {
    "description" => "Panel",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "panel",
    "type" => "string"
  },
  "permission" => {
    "description" => "Permission",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "permission",
    "type" => "string"
  },
  "phone_password_length" => {
    "description" => "Phone password length",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "phone_password_length",
    "type" => "string"
  },
  "primary_server" => {
    "description" => "Primary server",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "primary_server",
    "type" => "string"
  },
  "prompt_language" => {
    "description" => "Prompt language",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "prompt_language",
    "type" => "string"
  },
  "qualify" => {
    "description" => "Qualify",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "qualify",
    "type" => "string"
  },
  "schedule" => {
    "description" => "Schedule",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "schedule",
    "type" => "string"
  },
  "script" => {
    "description" => "Script",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "script",
    "type" => "string"
  },
  "secondary_server" => {
    "description" => "Secondary server",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "secondary_server",
    "type" => "string"
  },
  "status" => {
    "description" => "Status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "status",
    "type" => "string"
  },
  "tag" => {
    "description" => "Tag",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tag",
    "type" => "string"
  },
  "tenant" => {
    "description" => "Tenant",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant",
    "type" => "string"
  },
  "tenantprefix" => {
    "description" => "Tenantprefix",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenantprefix",
    "type" => "string"
  },
  "uappend" => {
    "description" => "Uappend",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "uappend",
    "type" => "string"
  },
  "url" => {
    "description" => "Url",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "url",
    "type" => "string"
  },
  "use_custom_logo" => {
    "description" => "Use custom logo",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "use_custom_logo",
    "type" => "string"
  },
  "usedistinctring" => {
    "description" => "Usedistinctring",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "usedistinctring",
    "type" => "string"
  },
  "users" => {
    "description" => "Users",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "users",
    "type" => "string"
  },
  "vm_options" => {
    "description" => "Vm options",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "vm_options",
    "type" => "string"
  },
  "web_password_length" => {
    "description" => "Web password length",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "web_password_length",
    "type" => "string"
  },
  "web_password_option" => {
    "description" => "Web password option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "web_password_option",
    "type" => "string"
  },
  "max_connect" => {
    "description" => "Thirdlane Connect enabled users limit",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxconnect",
    "type" => "string"
  },
  "connect_enabled" => {
    "description" => "Enable Thirdlane Connect",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "connect_enabled",
    "type" => "boolean"
  },
  "permission" => {
    "description" => "Permissions",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "vm_settings,call_forwarding,call_screening,call_blocking,allowrecording,contacts,enduser_company_contacts,crm,vm,recording_listen,call_history,preferences,integrations",
    "transform" => "permission",
    "type" => "string"
  },
  "permission" => {
    "description" => "Permissions",
    "format" => "raw",
    "hidden" => 0,
    "required" => 1,
    "sample" => "vm_settings,call_forwarding,call_screening,call_blocking,allowrecording,contacts,enduser_company_contacts,crm,vm,recording_listen,call_history,preferences,integrations",
    "default" => "vm_settings,call_forwarding,call_screening,call_blocking,allowrecording,contacts,enduser_company_contacts,crm,vm,recording_listen,call_history,preferences,integrations",
    "transform" => "permission",
    "type" => "string"
  }

);

my %in_hash_map = (
  "accountcode" => {
    "description" => "Accountcode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "accountcode",
    "type" => "string"
  },
  "allowaccountcode" => {
    "description" => "Allowaccountcode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "allowaccountcode",
    "type" => "string"
  },
  "allowcallerid" => {
    "description" => "Allowcallerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "allowcallerid",
    "type" => "string"
  },
  "attach" => {
    "description" => "Attach",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "attach",
    "type" => "string"
  },
  "call-limit" => {
    "description" => "Call-limit",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "call-limit",
    "type" => "string"
  },
  "callerid" => {
    "description" => "Callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid",
    "type" => "string"
  },
  "callerid_name" => {
    "description" => "Callerid name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "callerid_name",
    "type" => "string"
  },
  "canreinvite" => {
    "description" => "Canreinvite",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "canreinvite",
    "type" => "string"
  },
  "codec" => {
    "description" => "Codec",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "codec",
    "type" => "string"
  },
  "conf_announce_count" => {
    "description" => "Conf announce count",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_announce_count",
    "type" => "string"
  },
  "conf_announce_join" => {
    "description" => "Conf announce join",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_announce_join",
    "type" => "string"
  },
  "conf_detect_talker" => {
    "description" => "Conf detect talker",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_detect_talker",
    "type" => "string"
  },
  "conf_enable_moh" => {
    "description" => "Conf enable moh",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_enable_moh",
    "type" => "string"
  },
  "conf_maxusers" => {
    "description" => "Conf maxusers",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_maxusers",
    "type" => "string"
  },
  "conf_music" => {
    "description" => "Conf music",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_music",
    "type" => "string"
  },
  "conf_present_menu" => {
    "description" => "Conf present menu",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_present_menu",
    "type" => "string"
  },
  "conf_record" => {
    "description" => "Conf record",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_record",
    "type" => "string"
  },
  "conf_talker_optimization" => {
    "description" => "Conf talker optimization",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_talker_optimization",
    "type" => "string"
  },
  "conf_wait_marked" => {
    "description" => "Conf wait marked",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "conf_wait_marked",
    "type" => "string"
  },
  "contact_center" => {
    "description" => "Contact center",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "contact_center",
    "type" => "string"
  },
  "context" => {
    "description" => "Context",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "context",
    "type" => "string"
  },
  "create_conference" => {
    "description" => "Create conference",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_conference",
    "type" => "string"
  },
  "create_vm" => {
    "description" => "Create vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "create_vm",
    "type" => "string"
  },
  "crm_cidminlength" => {
    "description" => "Crm cidminlength",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidminlength",
    "type" => "string"
  },
  "crm_cidstrip" => {
    "description" => "Crm cidstrip",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_cidstrip",
    "type" => "string"
  },
  "crm_url_format" => {
    "description" => "Crm url format",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_url_format",
    "type" => "string"
  },
  "crm_use_defaults" => {
    "description" => "Crm use defaults",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "crm_use_defaults",
    "type" => "string"
  },
  "default_mailbox_pin" => {
    "description" => "Default mailbox pin",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "default_mailbox_pin",
    "type" => "string"
  },
  "default_web_password" => {
    "description" => "Default web password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "default_web_password",
    "type" => "string"
  },
  "delete" => {
    "description" => "Delete",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "delete",
    "type" => "string"
  },
  "description" => {
    "description" => "Description",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "name",
    "type" => "string"
  },
  "dtmfmode" => {
    "description" => "Dtmfmode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "dtmfmode",
    "type" => "string"
  },
  "emergency_callerid" => {
    "description" => "Emergency callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "emergency_callerid",
    "type" => "string"
  },
  "entered_other" => {
    "description" => "Entered other",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "entered_other",
    "type" => "string"
  },
  "entered_other_vm" => {
    "description" => "Entered other vm",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "entered_other_vm",
    "type" => "string"
  },
  "fax_default_paper_size" => {
    "description" => "Fax default paper size",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_default_paper_size",
    "type" => "string"
  },
  "fax_email_address" => {
    "description" => "Fax email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_email_address",
    "type" => "string"
  },
  "fax_maximum_number_of_retries" => {
    "description" => "Fax maximum number of retries",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_maximum_number_of_retries",
    "type" => "string"
  },
  "fax_notify_on_failure_and_success" => {
    "description" => "Fax notify on failure and success",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_notify_on_failure_and_success",
    "type" => "string"
  },
  "fax_retry_time" => {
    "description" => "Fax retry time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_retry_time",
    "type" => "string"
  },
  "fax_settings_station_id" => {
    "description" => "Fax settings station id",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_settings_station_id",
    "type" => "string"
  },
  "fax_wait_time" => {
    "description" => "Fax wait time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "fax_wait_time",
    "type" => "string"
  },
  "forwardemail" => {
    "description" => "Forwardemail",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardemail",
    "type" => "string"
  },
  "id" => {
    "description" => "Tenant's internal ID (autoincrement number)",
    "format" => "int64",
    "hidden" => 1,
    "required" => 0,
    "sample" => 1000,
    "transform" => "id",
    "type" => "integer"
  },
  "intalertinfo" => {
    "description" => "Intalertinfo",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "intalertinfo",
    "type" => "string"
  },
  "lang" => {
    "description" => "Lang",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lang",
    "type" => "string"
  },
  "mailbox_pin_length" => {
    "description" => "Mailbox pin length",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mailbox_pin_length",
    "type" => "string"
  },
  "mailbox_pin_option" => {
    "description" => "Mailbox pin option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mailbox_pin_option",
    "type" => "string"
  },
  "mappend" => {
    "description" => "Mappend",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mappend",
    "type" => "string"
  },
  "maxagents" => {
    "description" => "Maxagents",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxagents",
    "type" => "string"
  },
  "maxcalls" => {
    "description" => "Maxcalls",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxcalls",
    "type" => "string"
  },
  "maxconferences" => {
    "description" => "Maxconferences",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxconferences",
    "type" => "string"
  },
  "maxdevices" => {
    "description" => "Maxdevices",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxdevices",
    "type" => "string"
  },
  "maxmailboxes" => {
    "description" => "Maxmailboxes",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxmailboxes",
    "type" => "string"
  },
  "maxphones" => {
    "description" => "Maxphones",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxphones",
    "type" => "string"
  },
  "maxqueues" => {
    "description" => "Maxqueues",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxqueues",
    "type" => "string"
  },
  "maxusers" => {
    "description" => "Maxusers",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "maxusers",
    "type" => "string"
  },
  "model" => {
    "description" => "Model",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "model",
    "type" => "string"
  },
  "musiconhold" => {
    "description" => "Musiconhold",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "musiconhold",
    "type" => "string"
  },
  "nat" => {
    "description" => "Nat",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "nat",
    "type" => "string"
  },
  "officemode" => {
    "description" => "Officemode",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "officemode",
    "type" => "string"
  },
  "other" => {
    "description" => "Other",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other",
    "type" => "string"
  },
  "panel" => {
    "description" => "Panel",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "panel",
    "type" => "string"
  },
  "permission" => {
    "description" => "Permission",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "permission",
    "type" => "string"
  },
  "phone_password_length" => {
    "description" => "Phone password length",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "phone_password_length",
    "type" => "string"
  },
  "primary_server" => {
    "description" => "Primary server",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "primary_server",
    "type" => "string"
  },
  "prompt_language" => {
    "description" => "Prompt language",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "prompt_language",
    "type" => "string"
  },
  "qualify" => {
    "description" => "Qualify",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "qualify",
    "type" => "string"
  },
  "record" => {
    "description" => "Record",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD",
    "type" => "string"
  },
  "record_all" => {
    "description" => "Record all",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_ALL",
    "type" => "string"
  },
  "record_ext_in" => {
    "description" => "Record ext in",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_EXT_IN",
    "type" => "string"
  },
  "record_ext_out" => {
    "description" => "Record ext out",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_EXT_OUT",
    "type" => "string"
  },
  "record_int_in" => {
    "description" => "Record int in",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_INT_IN",
    "type" => "string"
  },
  "record_int_out" => {
    "description" => "Record int out",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_INT_OUT",
    "type" => "string"
  },
  "record_one_touch" => {
    "description" => "Record one touch",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_ONE_TOUCH",
    "type" => "string"
  },
  "schedule" => {
    "description" => "Schedule",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "schedule",
    "type" => "string"
  },
  "script" => {
    "description" => "Script",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "script",
    "type" => "string"
  },
  "secondary_server" => {
    "description" => "Secondary server",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "secondary_server",
    "type" => "string"
  },
  "status" => {
    "description" => "Status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "status",
    "type" => "string"
  },
  "tag" => {
    "description" => "Tag",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tag",
    "type" => "string"
  },
  "tenant" => {
    "description" => "Tenant",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant",
    "type" => "string"
  },
  "tenant_limits" => {
    "description" => "Tenant limits",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_limits",
    "type" => "string"
  },
  "tenant_menus" => {
    "description" => "Tenant menus",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_menus",
    "type" => "string"
  },
  "tenant_queues" => {
    "description" => "Tenant queues",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_queues",
    "type" => "string"
  },
  "tenant_routes" => {
    "description" => "Tenant routes",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_routes",
    "type" => "string"
  },
  "tenant_schedules" => {
    "description" => "Tenant schedules",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_schedules",
    "type" => "string"
  },
  "tenant_to_clone" => {
    "description" => "Tenant to clone",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_to_clone",
    "type" => "string"
  },
  "tenant_voiceprompts" => {
    "description" => "Tenant voiceprompts",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenant_voiceprompts",
    "type" => "string"
  },
  "tenantprefix" => {
    "description" => "Tenantprefix",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "tenantprefix",
    "type" => "string"
  },
  "uappend" => {
    "description" => "Uappend",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "uappend",
    "type" => "string"
  },
  "url" => {
    "description" => "Url",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "url",
    "type" => "string"
  },
  "use_custom_logo" => {
    "description" => "Use custom logo",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "use_custom_logo",
    "type" => "string"
  },
  "usedistinctring" => {
    "description" => "Usedistinctring",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "usedistinctring",
    "type" => "string"
  },
  "users" => {
    "description" => "Users",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "users",
    "type" => "string"
  },
  "vm_options" => {
    "description" => "Vm options",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "vm_options",
    "type" => "string"
  },
  "web_password_length" => {
    "description" => "Web password length",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "web_password_length",
    "type" => "string"
  },
  "web_password_option" => {
    "description" => "Web password option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "web_password_option",
    "type" => "string"
  },
  "maxconnect" => {
    "description" => "Thirdlane Connect enabled users limit",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "max_connect",
    "type" => "string"
  },
  "connect_enabled" => {
    "description" => "Enable Thirdlane Connect",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "connect_enabled",
    "type" => "boolean"
  },
  "permission" => {
    "description" => "Permissions",
    "format" => "raw",
    "hidden" => 0,
    "required" => 1,
    "sample" => "vm_settings,call_forwarding,call_screening,call_blocking,allowrecording,contacts,enduser_company_contacts,crm,vm,recording_listen,call_history,preferences,integrations",
    "default" => "vm_settings,call_forwarding,call_screening,call_blocking,allowrecording,contacts,enduser_company_contacts,crm,vm,recording_listen,call_history,preferences,integrations",
    "transform" => "permission",
    "type" => "string"
  }

);

our %requests = (
	types => [
	    {
		"type" => "tenant",
		"structure" => \%out_hash_map
	    }
	],
	operations => [
	    {
		"path" => "/api/tenants/",
		"method" => "GET",
		"summary" => "Get Tenants",
		"notes" => "Get all Tenants",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "tenant"
			}
		]
	    },
	    {
		"path" => "/api/tenants/{tenantName}",
		"method" => "GET",
		"summary" => "Get one Tenant",
		"notes" => "Get Tenant by name",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "thirdlane"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "tenant"
			}
		]
	    },
	    {
		"path" => "/api/tenants/",
		"method" => "POST",
		"summary" => "Add Tenant",
		"notes" => "Add Tenant",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "tenant",
			"parameter" => "request",
			"sample" => q({ "tenant" : "test1000",
			"callerid" : "14155551212",
			"emergency_callerid" : "14155551313",
			"tenant_to_clone" : "thirdlane",
			"tenant_limits" : "1",
			"tenant_routes" : "1",
			"tenant_schedules" : "1",
			"tenant_menus" : "1",
			"tenant_queues" : "1",
			"tenant_voiceprompts" :"1",
			"description" : "Test tenant" })
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "tenant"
			}
		]
	    },
	    {
		"path" => "/api/tenants/{tenantName}",
		"method" => "PUT",
		"summary" => "Update Tenant",
		"notes" => "Update Tenant",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "tenant",
			"parameter" => "request",
			"sample" => q({ "max_user_extensions" : "10" } )
		    },
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "test1000"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "tenant"
			}
		]
	    },
	    {
		"path" => "/api/tenants/{tenantName}",
		"method" => "DELETE",
		"summary" => "Delete Tenant",
		"notes" => "Delete Tenant",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "tenantName",
			"description" => "Tenant Name or Tenant ID",
			"required" => 1,
			"type" => "string",
			"parameter" => "path",
			"sample" => "test1000"
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "tenant"
			}
		]
	    }
	]
);

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}

sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{out} } ) { # fill default values
	    $out_hash->{$self->{map}->{out}->{$key}} = $self->{defaults}->{out}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{out}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{out}->{$key} && exists $hash->{$key};
    }

    $out_hash = $self->{mapper}->fix_type($out_hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $out_hash;
}

sub map_in {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{in} } ) { # fill default values
	    $out_hash->{$self->{map}->{in}->{$key}} = $self->{defaults}->{in}->{$key};
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	$out_hash->{$self->{map}->{in}->{$key}} = $hash->{$key}
	    if exists $self->{map}->{in}->{$key} && exists $hash->{$key};
    }

    return $out_hash;
}

1;
