package Asterisk::API::Structures::UserSettings;

use strict;
use warnings;


use JSON::XS;
use Types::Serialiser;

use Asterisk::API::Mapper::Generic;
use Asterisk::API::Documentation::Generic;

our $VERSION = "1";

our %out_hash_map = (
  "BLOCK" => {
    "description" => "BLOCK",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_status",
    "type" => "integer"
  },
  "BLOCKALL" => {
    "description" => "BLOCKALL",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_callerid_option",
    "type" => "integer"
  },
  "BLOCKDEST" => {
    "description" => "BLOCKDEST",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_action",
    "type" => "integer"
  },
  "BLOCKN" => {
    "description" => "BLOCKN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_block_callids_when_allowing",
    "type" => "string"
  },
  "BLOCKNCID" => {
    "description" => "BLOCKNCID",
    "format" => "integer",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_nocallerid_option",
    "type" => "integer"
  },
  "BLOCKOKN" => {
    "description" => "BLOCKOKN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_allow_callids_when_blocking",
    "type" => "string"
  },
  "BLOCK_SCHEDULE" => {
    "description" => "BLOCK SCHEDULE",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "blocking_time_range",
    "type" => "string"
  },
  "CFA" => {
    "description" => "CFA",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarding_always_status",
    "type" => "integer"
  },
  "CFAN" => {
    "description" => "CFAN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarding_always_number",
    "type" => "string"
  },
  "CFB" => {
    "description" => "CFB",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarding_busy_status",
    "type" => "integer"
  },
  "CFBN" => {
    "description" => "CFBN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarding_busy_number",
    "type" => "string"
  },
  "CFNA_SCHEDULE" => {
    "description" => "CFNA_SCHEDULE",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_cfna_schedule",
    "type" => "integer"
  },
  "CFA_SCHEDULE" => {
    "description" => "CFA_SCHEDULE",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_aschedule",
    "type" => "integer"
  },
  "CFB_SCHEDULE" => {
    "description" => "CFB_SCHEDULE",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_bschedule",
    "type" => "integer"
  },
  "CFCONFIRM" => {
    "description" => "CFCONFIRM",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_confirm_call_acceptance",
    "type" => "integer"
  },
  "CFNA" => {
    "description" => "CFNA",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_status",
    "type" => "integer"
  },
  "CFNAAN" => {
    "description" => "CFNAAN",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_announce_forwarding",
    "type" => "integer"
  },
  "CFNAEXT" => {
    "description" => "CFNAEXT",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_option",
    "type" => "integer"
  },
  "CFNAN1" => {
    "description" => "CFNAN1",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_number_1",
    "type" => "string"
  },
  "CFNAN2" => {
    "description" => "CFNAN2",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_number_2",
    "type" => "string"
  },
  "CFNAN3" => {
    "description" => "CFNAN3",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_number_3",
    "type" => "string"
  },
  "CFNAN4" => {
    "description" => "CFNAN4",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_number_4",
    "type" => "string"
  },
  "CFNAN5" => {
    "description" => "CFNAN5",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_number_5",
    "type" => "string"
  },
  "CFNAOKN" => {
    "description" => "CFNAOKN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_for_caller_ids",
    "type" => "string"
  },
  "CFNASTRAT" => {
    "description" => "CFNASTRAT",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_strategy",
    "type" => "integer"
  },
  "CFNAT1" => {
    "description" => "CFNAT1",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_time_1",
    "type" => "integer"
  },
  "CFNAT2" => {
    "description" => "CFNAT2",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_time_2",
    "type" => "integer"
  },
  "CFNAT3" => {
    "description" => "CFNAT3",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_time_3",
    "type" => "integer"
  },
  "CFNAT4" => {
    "description" => "CFNAT4",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_time_4",
    "type" => "integer"
  },
  "CFNAT5" => {
    "description" => "CFNAT5",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_sequential_time_5",
    "type" => "integer"
  },
  "CFNATALL" => {
    "description" => "CFNATALL",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_others_simultaneous_ring_time",
    "type" => "integer"
  },
  "CFNATGROUP" => {
    "description" => "CFNATGROUP",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_extension_and_others_ring_time",
    "type" => "integer"
  },
  "CFNATZERO" => {
    "description" => "CFNATZERO",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_original_extension_ring_time",
    "type" => "integer"
  },
  "CFRETAINCLI" => {
    "description" => "CFRETAINCLI",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_use_original_caller_id",
    "type" => "integer"
  },
  "RECORD" => {
    "description" => "RECORD",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "recording_status",
    "type" => "integer"
  },
  "RECORD_ALL" => {
    "description" => "RECORD ALL",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_all",
    "type" => "integer"
  },
  "RECORD_EXT_IN" => {
    "description" => "RECORD EXT IN",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_ext_in",
    "type" => "integer"
  },
  "RECORD_EXT_OUT" => {
    "description" => "RECORD EXT OUT",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_ext_out",
    "type" => "integer"
  },
  "RECORD_INT_IN" => {
    "description" => "RECORD INT IN",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_int_in",
    "type" => "integer"
  },
  "RECORD_INT_OUT" => {
    "description" => "RECORD INT OUT",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_int_out",
    "type" => "integer"
  },
  "RECORD_ONE_TOUCH" => {
    "description" => "RECORD ONE TOUCH",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "record_one_touch",
    "type" => "integer"
  },
  "SCREEN" => {
    "description" => "SCREEN",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "screening_status",
    "type" => "integer"
  },
  "SCREENENTER" => {
    "description" => "SCREENENTER",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "screening_nocallerid_enter_callerid",
    "type" => "integer"
  },
  "SCREENFOLLOW" => {
    "description" => "SCREENFOLLOW",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "screening_screen_while_in_findme",
    "type" => "integer"
  },
  "SCREENOKN" => {
    "description" => "SCREENOKN",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "screening_do_not_screen_callids",
    "type" => "string"
  },
  "SCREENVERBCID" => {
    "description" => "SCREENVERBCID",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "screening_callerid_announce",
    "type" => "integer"
  },
  "SCREENVERBNCID" => {
    "description" => "SCREENVERBNCID",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "screening_nocallerid_announce",
    "type" => "integer"
  },
  "VM" => {
    "description" => "VM",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_status",
    "type" => "integer"
  },
  "vm" => {
    "description" => "VM",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_status",
    "type" => "integer"
  },
  "VMTZERO" => {
    "description" => "VMTZERO",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_timeout",
    "type" => "integer"
  },
  "company" => {
    "description" => "Company",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "company",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "device_type" => {
    "description" => "Device type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "device_type",
    "type" => "string"
  },
  "email" => {
    "description" => "Email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_notify_email",
    "type" => "string"
  },
  "pager" => {
    "description" => "Email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_notify_email_second",
    "type" => "string"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "first_name",
    "type" => "string"
  },
  "forwardattach" => {
    "description" => "Forwardattach",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_notify_email_attach_messages",
    "type" => "boolean"
  },
  "forwarddelete" => {
    "description" => "Forwarddelete",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_delete_after_delivery",
    "type" => "boolean"
  },
  "forwardpager" => {
    "description" => "Forwardemail",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_notify_email_second_enabled",
    "type" => "boolean"
  },
  "forwardemail" => {
    "description" => "Forwardemail",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_notify_email_enabled",
    "type" => "boolean"
  },
  "home" => {
    "description" => "Home",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "home_phone",
    "type" => "string"
  },
  "image_url" => {
    "description" => "Image url",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "image_url",
    "type" => "string"
  },
  "lang" => {
    "description" => "Lang",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "language",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "last_name",
    "type" => "string"
  },
  "mobile" => {
    "description" => "Mobile",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile_phone",
    "type" => "string"
  },
  "office" => {
    "description" => "Office",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "office_phone",
    "type" => "string"
  },
  "other" => {
    "description" => "Other",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other_phone",
    "type" => "string"
  },
  "phone_password" => {
    "description" => "Phone password",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "phone_password",
    "type" => "string"
  },
  "pin" => {
    "description" => "Pin",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "voicemail_pin",
    "type" => "string"
  },
  "tz" => {
    "description" => "Timezone",
    "format" => "timezone",
    "hidden" => 0,
    "required" => 0,
    "sample" => "eastern",
    "transform" => "tz",
    "type" => "string"
  },
  "gcm_sender_id" => {
    "description" => "GCM Sender Id",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "1234567890",
    "transform" => "gcm_sender_id",
    "type" => "string"
  },
  "contact_email" => {
    "description" => "Contact Email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "contact_email",
    "type" => "string"
  }

);

my %in_hash_map = (
  "blocking_action" => {
    "description" => "Blocking action",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCKDEST",
    "type" => "string"
  },
  "blocking_allow_callids_when_blocking" => {
    "description" => "Blocking allow callids when blocking",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCKOKN",
    "type" => "string"
  },
  "blocking_block_callids_when_allowing" => {
    "description" => "Blocking block callids when allowing",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCKN",
    "type" => "string"
  },
  "blocking_callerid_option" => {
    "description" => "Blocking callerid option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCKALL",
    "type" => "string"
  },
  "blocking_nocallerid_option" => {
    "description" => "Blocking nocallerid option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCKNCID",
    "type" => "string"
  },
  "blocking_status" => {
    "description" => "Blocking status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCK",
    "type" => "string"
  },
  "blocking_time_range" => {
    "description" => "Blocking time range",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "BLOCK_SCHEDULE",
    "type" => "string"
  },
  "findme_cfna_schedule" => {
    "description" => "CFNA_SCHEDULE",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNA_SCHEDULE",
    "type" => "integer"
  },
  "CFA_SCHEDULE" => {
    "description" => "CFA_SCHEDULE",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_aschedule",
    "type" => "integer"
  },
  "CFB_SCHEDULE" => {
    "description" => "CFB_SCHEDULE",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "findme_bschedule",
    "type" => "integer"
  },
  "company" => {
    "description" => "Company",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "company",
    "type" => "string"
  },
  "department" => {
    "description" => "Department",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "department",
    "type" => "string"
  },
  "device_type" => {
    "description" => "Device type",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "device_type",
    "type" => "string"
  },
  "findme_announce_forwarding" => {
    "description" => "Findme announce forwarding",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAAN",
    "type" => "string"
  },
  "findme_confirm_call_acceptance" => {
    "description" => "Findme confirm call acceptance",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFCONFIRM",
    "type" => "string"
  },
  "findme_extension_and_others_ring_time" => {
    "description" => "Findme extension and others ring time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNATGROUP",
    "type" => "string"
  },
  "findme_for_caller_ids" => {
    "description" => "Findme for caller ids",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAOKN",
    "type" => "string"
  },
  "findme_option" => {
    "description" => "Findme option",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAEXT",
    "type" => "string"
  },
  "findme_original_extension_ring_time" => {
    "description" => "Findme original extension ring time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNATZERO",
    "type" => "string"
  },
  "findme_others_simultaneous_ring_time" => {
    "description" => "Findme others simultaneous ring time",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNATALL",
    "type" => "string"
  },
  "findme_sequential_number_1" => {
    "description" => "Findme sequential number 1",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAN1",
    "type" => "string"
  },
  "findme_sequential_number_2" => {
    "description" => "Findme sequential number 2",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAN2",
    "type" => "string"
  },
  "findme_sequential_number_3" => {
    "description" => "Findme sequential number 3",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAN3",
    "type" => "string"
  },
  "findme_sequential_number_4" => {
    "description" => "Findme sequential number 4",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAN4",
    "type" => "string"
  },
  "findme_sequential_number_5" => {
    "description" => "Findme sequential number 5",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAN5",
    "type" => "string"
  },
  "findme_sequential_time_1" => {
    "description" => "Findme sequential time 1",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAT1",
    "type" => "string"
  },
  "findme_sequential_time_2" => {
    "description" => "Findme sequential time 2",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAT2",
    "type" => "string"
  },
  "findme_sequential_time_3" => {
    "description" => "Findme sequential time 3",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAT3",
    "type" => "string"
  },
  "findme_sequential_time_4" => {
    "description" => "Findme sequential time 4",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAT4",
    "type" => "string"
  },
  "findme_sequential_time_5" => {
    "description" => "Findme sequential time 5",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNAT5",
    "type" => "string"
  },
  "findme_status" => {
    "description" => "Findme status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNA",
    "type" => "string"
  },
  "findme_strategy" => {
    "description" => "Findme strategy",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFNASTRAT",
    "type" => "string"
  },
  "findme_use_original_caller_id" => {
    "description" => "Findme use original caller id",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFRETAINCLI",
    "type" => "string"
  },
  "first_name" => {
    "description" => "First name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "first_name",
    "type" => "string"
  },
  "forwarding_always_number" => {
    "description" => "Forwarding always number",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFAN",
    "type" => "string"
  },
  "forwarding_always_status" => {
    "description" => "Forwarding always status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFA",
    "type" => "string"
  },
  "forwarding_busy_number" => {
    "description" => "Forwarding busy number",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFBN",
    "type" => "string"
  },
  "forwarding_busy_status" => {
    "description" => "Forwarding busy status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "CFB",
    "type" => "string"
  },
  "home_phone" => {
    "description" => "Home phone",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "home",
    "type" => "string"
  },
  "image_url" => {
    "description" => "Image url",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "image_url",
    "type" => "string"
  },
  "language" => {
    "description" => "Language",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "lang",
    "type" => "string"
  },
  "last_name" => {
    "description" => "Last name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "last_name",
    "type" => "string"
  },
  "mobile_phone" => {
    "description" => "Mobile phone",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "mobile",
    "type" => "string"
  },
  "office_phone" => {
    "description" => "Office phone",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "office",
    "type" => "string"
  },
  "other_phone" => {
    "description" => "Other phone",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "other",
    "type" => "string"
  },
  "record_all" => {
    "description" => "Record all",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_ALL",
    "type" => "string"
  },
  "record_ext_in" => {
    "description" => "Record ext in",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_EXT_IN",
    "type" => "string"
  },
  "record_ext_out" => {
    "description" => "Record ext out",
    "format" => "int64",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_EXT_OUT",
    "type" => "integer"
  },
  "record_int_in" => {
    "description" => "Record int in",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_INT_IN",
    "type" => "string"
  },
  "record_int_out" => {
    "description" => "Record int out",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_INT_OUT",
    "type" => "string"
  },
  "record_one_touch" => {
    "description" => "Record one touch",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD_ONE_TOUCH",
    "type" => "string"
  },
  "recording_status" => {
    "description" => "Recording status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "RECORD",
    "type" => "string"
  },
  "screening_callerid_announce" => {
    "description" => "Screening callerid announce",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "SCREENVERBCID",
    "type" => "string"
  },
  "screening_do_not_screen_callids" => {
    "description" => "Screening do not screen callids",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "SCREENOKN",
    "type" => "string"
  },
  "screening_nocallerid_announce" => {
    "description" => "Screening nocallerid announce",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "SCREENVERBNCID",
    "type" => "string"
  },
  "screening_nocallerid_enter_callerid" => {
    "description" => "Screening nocallerid enter callerid",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "SCREENENTER",
    "type" => "string"
  },
  "screening_screen_while_in_findme" => {
    "description" => "Screening screen while in findme",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "SCREENFOLLOW",
    "type" => "string"
  },
  "screening_status" => {
    "description" => "Screening status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "SCREEN",
    "type" => "string"
  },
  "tag" => {
    "description" => "Tag name",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "transform" => "tag",
    "type" => "string"
  },
  "tz" => {
    "description" => "Timezone",
    "format" => "timezone",
    "hidden" => 0,
    "required" => 0,
    "sample" => "eastern",
    "transform" => "tz",
    "type" => "string"
  },
  "voicemail_delete_after_delivery" => {
    "description" => "Voicemail delete after delivery",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwarddelete",
    "type" => "string"
  },
  "voicemail_notify_email" => {
    "description" => "Voicemail notify email",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "email",
    "type" => "string"
  },
  "voicemail_notify_email_attach_messages" => {
    "description" => "Voicemail notify email attach messages",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardattach",
    "type" => "string"
  },
  "voicemail_notify_email_enabled" => {
    "description" => "Voicemail notify email enabled",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardemail",
    "type" => "string"
  },
  "voicemail_notify_email_second_enabled" => {
    "description" => "Voicemail notify second email enabled",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "forwardpager",
    "type" => "string"
  },
  "voicemail_notify_email_second" => {
    "description" => "Voicemail notify email second",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pager",
    "type" => "string"
  },
  "voicemail_pin" => {
    "description" => "Voicemail pin",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "pin",
    "type" => "string"
  },
  "voicemail_status" => {
    "description" => "Voicemail status",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => [ "VM", "vm" ],
    "type" => "string"
  },
  "voicemail_timeout" => {
    "description" => "Voicemail timeout",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "VMTZERO",
    "type" => "string"
  },
  "contact_email" => {
    "description" => "Contact Email address",
    "format" => "raw",
    "hidden" => 0,
    "required" => 0,
    "sample" => "",
    "transform" => "contact_email",
    "type" => "string"
  }

);

our %requests = (
	types => [
	    {
		"type" => "user_settings",
		"structure" => \%out_hash_map
	    }
	],
	operations => [
	    {
		"path" => "/api/user/settings",
		"method" => "GET",
		"summary" => "Get User Settings",
		"notes" => "Get User Settings",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "user_settings"
			}
		]
	    },
	    {
		"path" => "/api/user/settings",
		"method" => "PUT",
		"summary" => "Update User Settings",
		"notes" => "Update User Settings",
		"consumes" => [
			"application/json"
		],
		"parameters" => [
		    {
			"name" => "body",
			"description" => "No description",
			"required" => 1,
			"type" => "user_settings",
			"parameter" => "request",
			"sample" => q({"VM" :"4" })
		    }
		],
		"responses" => [
			{
				"code" => 200,
				"type" => "user_settings"
			}
		]
	    }
	]
);

sub new {
    my( $class, $args ) = @_;

    my $self = bless( { }, $class );

    $self->{version} = $VERSION;

    $self->{mapper} = new Asterisk::API::Mapper::Generic;

    my $out_mapping = $self->{mapper}->prepare( \%out_hash_map );
    $self->{defaults}->{out} = $out_mapping->{defaults};
    $self->{map}->{out} = $out_mapping->{map};
    $self->{type}->{out} = $out_mapping->{type};
    $self->{reverse}->{out} = $out_mapping->{reverse};

    my $in_mapping = $self->{mapper}->prepare( \%in_hash_map );
    $self->{defaults}->{in} = $in_mapping->{defaults};
    $self->{map}->{in} = $in_mapping->{map};
    $self->{type}->{in} = $in_mapping->{type};
    $self->{reverse}->{in} = $in_mapping->{reverse};

    return $self;
}


sub info {
    my ( $self ) = @_;

    my $doc = new Asterisk::API::Documentation::Generic;

    return $doc->prepare_types( \%requests );
}

sub map_out {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{out} } ) { # fill default values
	    if ( exists $self->{map}->{out}->{$key} ) {
		if ( ref $self->{map}->{out}->{$key} eq "ARRAY" ) {
		    foreach my $transform_key ( @{ $self->{map}->{out}->{$key} } ) {
			$out_hash->{$transform_key} = $self->{defaults}->{out}->{$key};
		    }
		}
		else {
		    $out_hash->{$self->{map}->{out}->{$key}} = $self->{defaults}->{out}->{$key};
		}
	    }
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	if ( exists $self->{map}->{out}->{$key} && exists $hash->{$key} ) {
	    if ( ref $self->{map}->{out}->{$key} eq "ARRAY" ) {
		foreach my $transform_key ( @{ $self->{map}->{out}->{$key} } ) {
		    $out_hash->{$transform_key} = $hash->{$key};
		}
	    }
	    else {
		$out_hash->{$self->{map}->{out}->{$key}} = $hash->{$key};
	    }
	}
    }

    $out_hash = $self->{mapper}->fix_type($out_hash, $self->{type}->{out}, $self->{reverse}->{out});

    return $out_hash;
}


sub map_in {
    my ( $self, $hash, $params ) = @_;

    if ( exists $params->{direct} && $params->{direct} ) {
	return $hash;
    }

    my $out_hash = { };

    if ( exists $params->{defaults} && $params->{defaults} ) {
	foreach my $key ( keys %{ $self->{defaults}->{in} } ) { # fill default values
	    if ( exists $self->{map}->{in}->{$key} ) {
		if ( ref $self->{map}->{in}->{$key} eq "ARRAY" ) {
		    foreach my $transform_key ( @{ $self->{map}->{in}->{$key} } ) {
			$out_hash->{$transform_key} = $self->{defaults}->{in}->{$key};
		    }
		}
		else {
		    $out_hash->{$self->{map}->{in}->{$key}} = $self->{defaults}->{in}->{$key};
		}
	    }
	}
    }

    foreach my $key ( keys %{ $hash } ) {
	if ( exists $self->{map}->{in}->{$key} && exists $hash->{$key} ) {
	    if ( ref $self->{map}->{in}->{$key} eq "ARRAY" ) {
		foreach my $transform_key ( @{ $self->{map}->{in}->{$key} } ) {
		    $out_hash->{$transform_key} = $hash->{$key};
		}
	    }
	    else {
		$out_hash->{$self->{map}->{in}->{$key}} = $hash->{$key};
	    }
	}
    }

    return $out_hash;
}

1;
