package Integration::CRM;

use utf8;
use strict;
use warnings;

use Integration::CRM::SalesForce;
#use Integration::CRM::Zoho;
use Integration::CRM::Sugar;
use Integration::CRM::VTiger;
use Integration::CRM::X2CRM;
use Integration::CRM::Freshdesk;
use Integration::CRM::PipeDrive;
#use Integration::CRM::CLI;

use lib '/opt/thirdlane/perl_lib/lib';

use Thirdlane::Auth::OAuth2;
use Integration::Contact::Source::Google::Contacts;
use Integration::Contact::Source::Microsoft::Contacts;
use Integration::Contact::Source::Zoho::Contacts;

our $VERSION = '1';

sub new {
    my( $class, $args ) = @_;
    my $self = bless( { }, $class );
    $self->{config} = $args;
    $self->{version} = $VERSION;

    $self->{id} = $args->{id};
    $self->{type} = $args->{type};

    if ( $args->{type} eq 'salesforce') {
	$self->{crm} = Integration::CRM::SalesForce->new($args);
    }

#    if ( $args->{type} eq 'zoho') {
#	$self->{crm} = Integration::CRM::Zoho->new($args);
#    }

    elsif ( $args->{type} eq 'sugar') {
	$self->{crm} = Integration::CRM::Sugar->new($args);
    }

    elsif ( $args->{type} eq 'vtiger') {
	$self->{crm} = Integration::CRM::VTiger->new($args);
    }

    elsif ( $args->{type} eq 'x2crm') {
	$self->{crm} = Integration::CRM::X2CRM->new($args);
    }

    elsif ( $args->{type} eq 'freshdesk') {
	$self->{crm} = Integration::CRM::Freshdesk->new($args);
    }

    elsif ( $args->{type} eq 'pipedrive') {
	$self->{crm} = Integration::CRM::PipeDrive->new($args);
    }

    elsif ( lc $args->{type} eq 'google') {
	$self->{crm} = Integration::Contact::Source::Google::Contacts->new($args);
    }

    elsif ( lc $args->{type} eq 'microsoft') {
	$self->{crm} = Integration::Contact::Source::Microsoft::Contacts->new($args);
    }

    elsif ( lc $args->{type} eq 'zoho') {
	$self->{crm} = Integration::Contact::Source::Zoho::Contacts->new($args);
    }

#   elsif ( $args->{type} eq 'cli') {
#	$self->{crm} = Integration::CRM::CLI->new($args);
#    }

    die "crm is not supported" unless $self->{crm};

    return $self;
}

sub crm {
    my $self = shift;
    return $self->{crm};
}

sub detect_type_by_id {
    my ( $self, $id ) = @_;
    return $self->crm->detect_type_by_id($id);
}

sub get_field_list {
    my ( $self, $object ) = @_;
    return $self->crm->get_field_list($object);
}

sub get_fields {
    my ( $self, $object ) = @_;
    return $self->crm->detect_type_by_id($object);
}

sub _connect {
    my $self = shift;
    return $self->crm->_connect();
}

sub upsert {
    my( $self, $args ) = @_;
    return $self->crm->upsert($args);
}

sub find {
    my( $self, $args ) = @_;
    return $self->crm->find($args);
}

sub query {
    my( $self, $args ) = @_;
    return $self->crm->query($args);
}

sub find_phone {
    my( $self, $phone ) = @_;
    return $self->crm->find_phone($phone);
}

sub query_tasks_by_rec_id {
    my( $self, $rec_id ) = @_;
    return $self->crm->query_tasks_by_rec_id($rec_id);
}

sub query_tasks_by_id {
    my( $self, $id ) = @_;
    return $self->crm->query_tasks_by_id($id);
}

sub query_leads_by_id {
    my( $self, $id ) = @_;
    return $self->crm->query_leads_by_id($id);
}

sub query_contacts_by_id {
    my( $self, $id ) = @_;
    return $self->crm->query_contacts_by_id($id);
}

sub query_accounts_by_id {
    my( $self, $id ) = @_;
    return $self->crm->query_accounts_by_id($id);
}

sub query_all {
    my( $self, $query, $fields ) = @_;

    my $result = [ ];

    $result = $self->crm->query_all($query, $fields);

    return $result;
}

1;
