package Thirdlane::Cache;
use base qw(CHI);

use utf8;
use strict;
use warnings;

use lib '/opt/thirdlane/perl_lib/local/lib/perl5';


use File::Spec;
use File::Temp;
use File::Path qw(make_path remove_tree);

use constant {
    COMPRESS_CACHE => 0,
    SERIALIZER => 'MessagePack',
    KEY_DIGESTER => 'MD5',
    COMPRESSOR => 'Compress::Zlib',
    CACHE_ROOT_DIR => '/var/cache/thirdlane',
    ENCRYPT_CIPHER => 'Blowfish',
    ENCRYPT_SECRET => 'temp_secret'
};

use constant {
    FILE_BACKEND_ROOT => File::Spec->catdir( CACHE_ROOT_DIR, 'filemap' ),
    FMMAP_BACKEND_ROOT => File::Spec->catdir( CACHE_ROOT_DIR, 'fffmap' ),
    TEMP_BACKEND_ROOT => File::Spec->catdir( CACHE_ROOT_DIR, 'temp' )
};

use constant {
    MEMCACHED_SERVERS => [ '127.0.0.1:11211' ],
    MEMCACHED_COMPRESS_THREADSHOLD => 10_000,
    ROOT_PATH_CREATED => scalar make_path(FILE_BACKEND_ROOT),
    TEMP_ROOT_PATH_CREATED => scalar make_path(TEMP_BACKEND_ROOT),
    FMMAP_ROOT_PATH_CREATED => scalar make_path(FMMAP_BACKEND_ROOT)
};

sub rand_str { join("",sort{rand(1024)<=>rand(1024)}do{my@rs=('A'..'Z','a'..'z',0..9);srand(time()*rand(time()));map {my$cr;do{$cr=$rs[rand(1024)]}until($cr);$cr}1..32}); }

__PACKAGE__->config({
    storage => {
	file_storage => {
	    driver => 'File',
	    root_dir => sprintf( "%s", File::Temp->newdir( TEMPLATE => "fcache-XXXXXX", DIR => TEMP_BACKEND_ROOT ) ),
	    depth          => 4,
	    max_key_length => 248
	},
	file_storage_persistent => {
	    driver => 'File',
	    root_dir => make_path(FILE_BACKEND_ROOT) ? FILE_BACKEND_ROOT : FILE_BACKEND_ROOT,
	    depth          => 4,
	    max_key_length => 248
	},
	memcached_storage  => {
	    driver => 'Memcached::Fast::Safe',
	    servers => MEMCACHED_SERVERS,
	    compress_threshold => MEMCACHED_COMPRESS_THREADSHOLD,
	},
	memory_storage_local => {
	    driver => 'Memory',
	    global => 0
	},
	memory_storage_global => {
	    driver => 'Memory',
	    global => 1
	},
	mmap_storage => {
	    driver => 'FastMmap',
	    root_dir => sprintf( "%s", File::Temp->newdir( TEMPLATE => "mmapcache-XXXXXX", DIR => TEMP_BACKEND_ROOT ) ),
	},
	mmap_storage_persistent => {
	    driver => 'FastMmap',
	    root_dir => make_path(FMMAP_BACKEND_ROOT) ? FMMAP_BACKEND_ROOT : FMMAP_BACKEND_ROOT,
	}
    },
    namespace => {
	'Default' => {
	    storage => 'memcached_storage',
	},
	'Storage::Memory' => {
	    storage => 'memory_storage_local'
	},
	'Storage::Memory::Memcached' => {
	    storage => 'memcached_storage'
	},
	'Storage::Memory::Memcached::Encrypted' => {
	    storage => 'memcached_storage',
	    serializer => {
		serializer => SERIALIZER,
		digester   => KEY_DIGESTER,
		compressor => COMPRESSOR,
		compress   => COMPRESS_CACHE,
		cipher     => ENCRYPT_CIPHER,
		secret     => ENCRYPT_SECRET,
		serializer_token => 1
	    }
	},
	'Storage::File::Encrypted' => {
	    storage => 'file_storage_persistent',
	    serializer => {
		serializer => SERIALIZER,
		digester   => KEY_DIGESTER,
		compressor => COMPRESSOR,
		compress   => COMPRESS_CACHE,
		cipher     => ENCRYPT_CIPHER,
		secret     => ENCRYPT_SECRET,
		serializer_token => 1
	    }
	},
	'Storage::File::Temp' => {
	    storage => 'file_storage',
	    namespace => rand_str,
	    root_dir => File::Temp->newdir( TEMPLATE => "tlcache-XXXXXX", DIR => TEMP_BACKEND_ROOT )
	},
	'Storage::File::Temp::Encrypted' => {
	    storage => 'file_storage',
	    namespace => rand_str,
	    root_dir => File::Temp->newdir( TEMPLATE => "tlcache-XXXXXX", DIR => TEMP_BACKEND_ROOT ),
	    serializer => {
		serializer => SERIALIZER,
		digester   => KEY_DIGESTER,
		compressor => COMPRESSOR,
		compress   => COMPRESS_CACHE,
		cipher     => ENCRYPT_CIPHER,
		secret     => ENCRYPT_SECRET,
		serializer_token => 1
	    }
	},
	'Storage::Config' => {
	    storage => 'mmap_storage_persistent',
	    root_dir => make_path(FMMAP_BACKEND_ROOT) ? FMMAP_BACKEND_ROOT : FMMAP_BACKEND_ROOT
	},
	'Storage::Config::Encrypted' => {
	    storage => 'mmap_storage_persistent',
	    root_dir => make_path(FMMAP_BACKEND_ROOT) ? FMMAP_BACKEND_ROOT : FMMAP_BACKEND_ROOT,
	    serializer => {
		serializer => SERIALIZER,
		digester   => KEY_DIGESTER,
		compressor => COMPRESSOR,
		compress   => COMPRESS_CACHE,
		cipher     => ENCRYPT_CIPHER,
		secret     => ENCRYPT_SECRET,
		serializer_token => 1
	    }
	},
	'Storage::Config::Temp::Encrypted' => {
	    storage => 'mmap_storage',
	    root_dir => File::Temp->newdir( TEMPLATE => "tlcache-XXXXXX", DIR => TEMP_BACKEND_ROOT ),
	    serializer => {
		serializer => SERIALIZER,
		digester   => KEY_DIGESTER,
		compressor => COMPRESSOR,
		compress   => COMPRESS_CACHE,
		cipher     => ENCRYPT_CIPHER,
		secret     => ENCRYPT_SECRET,
		serializer_token => 1
	    }
	}
    },
    defaults  => {
	storage => 'memory_storage_local',
	serializer => {
	    serializer => SERIALIZER,
	    digester   => KEY_DIGESTER,
	    compressor => COMPRESSOR,
	    serializer_token => 1,
	    compress   => COMPRESS_CACHE
	}
    }
});

1;
